import { reactive } from 'vue'

/**
 * Generic optimistic-update helper.
 *
 * Usage:
 *  const { apply, confirm, rollback } = useOptimisticUpdates()
 *  apply(id, () => { item.done = true }, () => { item.done = false })
 *  await apiCall()
 *  confirm(id) // or rollback(id) on failure
 */
export function useOptimisticUpdates() {
    // Map of rollback callbacks keyed by mutation id
    const pending = reactive(new Map<string, () => void>())

    function apply(id: string, mutate: () => void, rollback: () => void) {
        if (pending.has(id)) return
        mutate()
        pending.set(id, rollback)
    }

    function confirm(id: string) {
        pending.delete(id)
    }

    function rollback(id: string) {
        const fn = pending.get(id)
        if (fn) fn()
        pending.delete(id)
    }

    return { apply, confirm, rollback }
} 