import { ref, computed, onMounted, watch } from 'vue'
import { useChoreStore } from '@/stores/choreStore'
import { useExpenses } from '@/composables/useExpenses'
import { useGroupStore } from '@/stores/groupStore'
import { useAuthStore } from '@/stores/auth'
import type { Chore } from '@/types/chore'

interface NextAction {
    type: 'chore' | 'expense' | 'none';
    title: string;
    subtitle: string;
    cta: string;
    path: string;
    priority: number; // 1 = highest
}

export function usePersonalStatus() {
    const choreStore = useChoreStore()
    const groupStore = useGroupStore()
    const { expenses, fetchExpenses } = useExpenses()
    const authStore = useAuthStore()

    const isLoading = ref(true)

    const fetchAllData = async (groupId: number) => {
        isLoading.value = true
        await Promise.all([
            choreStore.fetchGroup(groupId),
            choreStore.fetchPersonal(),
            fetchExpenses({ group_id: groupId }),
        ])
        isLoading.value = false
    }

    onMounted(() => {
        if (groupStore.currentGroupId) {
            fetchAllData(groupStore.currentGroupId)
        }
    })

    watch(() => groupStore.currentGroupId, (newGroupId) => {
        if (newGroupId) {
            fetchAllData(newGroupId)
        }
    })

    const userChoresWithAssignments = computed(() => {
        const userId = authStore.user?.id
        if (!userId) return []

        return choreStore.allChores.flatMap(chore =>
            chore.assignments
                .filter(assignment => assignment.assigned_to_user_id === userId)
                .map(assignment => ({
                    choreName: chore.name,
                    ...assignment
                }))
        )
    })

    const nextAction = computed<NextAction>(() => {
        const now = new Date()
        // Priority 1: Overdue chores
        const overdueChoreAssignment = userChoresWithAssignments.value
            .filter(assignment => new Date(assignment.due_date) < now && !assignment.is_complete)
            .sort((a, b) => new Date(a.due_date).getTime() - new Date(b.due_date).getTime())[0]

        if (overdueChoreAssignment) {
            return {
                type: 'chore',
                title: `Overdue: ${overdueChoreAssignment.choreName}`,
                subtitle: `Was due on ${new Date(overdueChoreAssignment.due_date).toLocaleDateString()}`,
                cta: 'View Chore',
                path: `/chores`,
                priority: 1,
            }
        }

        // Placeholder for expense logic - to be implemented

        // Priority 2: Upcoming chores
        const upcomingChoreAssignment = userChoresWithAssignments.value
            .filter(assignment => !assignment.is_complete)
            .sort((a, b) => new Date(a.due_date).getTime() - new Date(b.due_date).getTime())[0]

        if (upcomingChoreAssignment) {
            return {
                type: 'chore',
                title: `Next up: ${upcomingChoreAssignment.choreName}`,
                subtitle: `Due on ${new Date(upcomingChoreAssignment.due_date).toLocaleDateString()}`,
                cta: 'View Chore',
                path: `/chores`,
                priority: 2,
            }
        }

        return {
            type: 'none',
            title: "You're all caught up!",
            subtitle: 'No urgent tasks or expenses. Great job!',
            cta: 'Go to Dashboard',
            path: '/dashboard',
            priority: 99,
        }
    })

    return {
        nextAction,
        isLoading,
    }
} 