import { ref, computed } from 'vue'
import { defineStore } from 'pinia'
import { activityService } from '@/services/activityService'
import { API_BASE_URL } from '@/config/api-config'
import type { Activity } from '@/types/activity'

export const useActivityStore = defineStore('activity', () => {
    const activities = ref<Activity[]>([])
    const isLoading = ref(false)
    const error = ref<string | null>(null)
    const cursor = ref<number | null>(null)
    const hasMore = ref(true)
    const currentGroupId = ref<number | null>(null)
    const socket = ref<WebSocket | null>(null)

    async function fetchActivities(groupId: number, limit: number = 20) {
        if (isLoading.value || !hasMore.value) return;

        isLoading.value = true
        error.value = null

        if (groupId !== currentGroupId.value) {
            // Reset if group changes
            activities.value = []
            cursor.value = null
            hasMore.value = true
        }
        currentGroupId.value = groupId

        try {
            const response = await activityService.getGroupActivity(groupId, limit, cursor.value)
            activities.value.push(...response.items)
            cursor.value = response.cursor
            if (!response.cursor || response.items.length < limit) {
                hasMore.value = false
            }
        } catch (e: any) {
            error.value = e.message || 'Failed to fetch activities'
        } finally {
            isLoading.value = false
        }
    }

    function buildWsUrl(groupId: number, token: string | null): string {
        const base = import.meta.env.DEV ? `ws://${location.host}/ws/${groupId}` : `${API_BASE_URL.replace('https', 'wss').replace('http', 'ws')}/ws/${groupId}`
        return token ? `${base}?token=${token}` : base
    }

    function connectWebSocket(groupId: number, token: string | null) {
        if (socket.value) {
            socket.value.close()
        }
        const url = buildWsUrl(groupId, token)
        socket.value = new WebSocket(url)
        socket.value.onopen = () => {
            console.debug('[WS] Connected to', url)
        }
        socket.value.onmessage = (event) => {
            try {
                const data = JSON.parse(event.data)
                if (data && data.event_type) {
                    activities.value.unshift(data as Activity)
                }
            } catch (e) {
                console.error('[WS] failed to parse message', e)
            }
        }
        socket.value.onclose = () => {
            console.debug('[WS] closed')
            socket.value = null
        }
        socket.value.onerror = (e) => {
            console.error('[WS] error', e)
        }
    }

    function disconnectWebSocket() {
        if (socket.value) {
            socket.value.close()
            socket.value = null
        }
    }

    return {
        activities,
        isLoading,
        error,
        hasMore,
        fetchActivities,
        connectWebSocket,
        disconnectWebSocket,
    }
}) 