import { defineStore } from 'pinia'
import { ref, computed } from 'vue'
import type { Chore, ChoreCreate, ChoreUpdate, ChoreType, ChoreAssignment } from '@/types/chore'
import { choreService } from '@/services/choreService'
import { apiClient, API_ENDPOINTS } from '@/services/api'
import type { TimeEntry } from '@/types/time_entry'

export const useChoreStore = defineStore('chores', () => {
    // ---- State ----
    const personal = ref<Chore[]>([])
    const groupById = ref<Record<number, Chore[]>>({})
    const isLoading = ref(false)
    const error = ref<string | null>(null)
    const timeEntriesByAssignment = ref<Record<number, TimeEntry[]>>({})

    // ---- Getters ----
    const allChores = computed<Chore[]>(() => [
        ...personal.value,
        ...Object.values(groupById.value).flat(),
    ])

    const activeTimerEntry = computed<TimeEntry | null>(() => {
        for (const assignmentId in timeEntriesByAssignment.value) {
            const entry = timeEntriesByAssignment.value[assignmentId].find((te) => !te.end_time);
            if (entry) return entry;
        }
        return null;
    });

    function setError(message: string) {
        error.value = message
        isLoading.value = false
    }

    // ---- Actions ----
    async function fetchPersonal() {
        isLoading.value = true
        error.value = null
        try {
            personal.value = await choreService.getPersonalChores()
        } catch (e: any) {
            setError(e?.message || 'Failed to load personal chores')
        } finally {
            isLoading.value = false
        }
    }

    async function fetchGroup(groupId: number) {
        isLoading.value = true
        error.value = null
        try {
            const chores = await choreService.getChores(groupId)
            groupById.value[groupId] = chores
        } catch (e: any) {
            setError(e?.message || 'Failed to load group chores')
        } finally {
            isLoading.value = false
        }
    }

    async function create(chore: ChoreCreate) {
        try {
            const created = await choreService.createChore(chore)
            if (created.type === 'personal') {
                personal.value.push(created)
            } else if (created.group_id != null) {
                if (!groupById.value[created.group_id]) groupById.value[created.group_id] = []
                groupById.value[created.group_id].push(created)
            }
            return created
        } catch (e: any) {
            setError(e?.message || 'Failed to create chore')
            throw e
        }
    }

    async function update(choreId: number, updates: ChoreUpdate, original: Chore) {
        try {
            const updated = await choreService.updateChore(choreId, updates, original)
            // Remove from previous list
            if (original.type === 'personal') {
                personal.value = personal.value.filter((c) => c.id !== choreId)
            } else if (original.group_id != null) {
                groupById.value[original.group_id] = (groupById.value[original.group_id] || []).filter((c) => c.id !== choreId)
            }
            // Add to new list
            if (updated.type === 'personal') {
                personal.value.push(updated)
            } else if (updated.group_id != null) {
                if (!groupById.value[updated.group_id]) groupById.value[updated.group_id] = []
                groupById.value[updated.group_id].push(updated)
            }
            return updated
        } catch (e: any) {
            setError(e?.message || 'Failed to update chore')
            throw e
        }
    }

    async function remove(choreId: number, choreType: ChoreType, groupId?: number) {
        try {
            await choreService.deleteChore(choreId, choreType, groupId)
            if (choreType === 'personal') {
                personal.value = personal.value.filter((c) => c.id !== choreId)
            } else if (groupId != null) {
                groupById.value[groupId] = (groupById.value[groupId] || []).filter((c) => c.id !== choreId)
            }
        } catch (e: any) {
            setError(e?.message || 'Failed to delete chore')
            throw e
        }
    }

    async function fetchTimeEntries(assignmentId: number) {
        try {
            const response = await apiClient.get(`${API_ENDPOINTS.CHORES.ASSIGNMENT_BY_ID(assignmentId)}/time-entries`)
            timeEntriesByAssignment.value[assignmentId] = response.data
        } catch (e: any) {
            setError(e?.message || `Failed to fetch time entries for assignment ${assignmentId}`)
        }
    }

    async function startTimer(assignmentId: number) {
        try {
            const response = await apiClient.post(`${API_ENDPOINTS.CHORES.ASSIGNMENT_BY_ID(assignmentId)}/time-entries`)
            if (!timeEntriesByAssignment.value[assignmentId]) {
                timeEntriesByAssignment.value[assignmentId] = []
            }
            timeEntriesByAssignment.value[assignmentId].push(response.data)
            return response.data
        } catch (e: any) {
            setError(e?.message || `Failed to start timer for assignment ${assignmentId}`)
            throw e
        }
    }

    async function stopTimer(assignmentId: number, timeEntryId: number) {
        try {
            const response = await apiClient.put(`${API_ENDPOINTS.CHORES.TIME_ENTRY(timeEntryId)}`)
            const entries = timeEntriesByAssignment.value[assignmentId] || []
            const index = entries.findIndex((t) => t.id === timeEntryId)
            if (index > -1) {
                entries[index] = response.data
            }
            return response.data
        } catch (e: any) {
            setError(e?.message || `Failed to stop timer for entry ${timeEntryId}`)
            throw e
        }
    }

    async function toggleCompletion(chore: Chore, assignment: ChoreAssignment | null) {
        try {
            let currentAssignment = assignment;
            if (!currentAssignment) {
                // create assignment default user
                currentAssignment = await choreService.createAssignment({
                    chore_id: chore.id,
                    assigned_to_user_id: chore.created_by_id,
                    due_date: chore.next_due_date,
                });
            }
            const newStatus = !currentAssignment.is_complete;
            let updatedAssignment: ChoreAssignment;
            if (newStatus) {
                updatedAssignment = await choreService.completeAssignment(currentAssignment.id);
            } else {
                updatedAssignment = await choreService.updateAssignment(currentAssignment.id, { is_complete: false });
            }
            // update local state: find chore and update assignment status.
            function applyToList(list: Chore[]) {
                const cIndex = list.findIndex((c) => c.id === chore.id);
                if (cIndex === -1) return;
                const c = list[cIndex];
                const aIndex = c.assignments.findIndex((a) => a.id === updatedAssignment.id);
                if (aIndex !== -1) {
                    c.assignments[aIndex] = updatedAssignment;
                }
            }
            if (chore.type === 'personal') {
                applyToList(personal.value);
            } else if (chore.group_id != null) {
                applyToList(groupById.value[chore.group_id] || []);
            }
            return updatedAssignment;
        } catch (e: any) {
            setError(e?.message || 'Failed to toggle completion');
            throw e;
        }
    }

    return {
        personal,
        groupById,
        allChores,
        isLoading,
        error,
        fetchPersonal,
        fetchGroup,
        create,
        update,
        remove,
        setError,
        timeEntriesByAssignment,
        activeTimerEntry,
        fetchTimeEntries,
        startTimer,
        stopTimer,
        toggleCompletion
    }
}) 