import { defineStore } from 'pinia'
import { ref } from 'vue'
import type { Item } from '@/types/item'
import { apiClient, API_ENDPOINTS } from '@/services/api'

interface CreateItemPayload {
    listId: number
    data: {
        name: string
        quantity?: string | number | null
        category_id?: number | null
    }
}

interface UpdateItemPayload {
    listId: number
    itemId: number
    data: Partial<
        Omit<
            Item,
            | 'id'
            | 'list_id'
            | 'added_by_id'
            | 'completed_by_id'
            | 'added_by_user'
            | 'completed_by_user'
            | 'created_at'
            | 'updated_at'
        > & { version: number }
    >
}

export const useItemStore = defineStore('items', () => {
    const itemsByList = ref<Record<number, Item[]>>({})
    const isLoading = ref(false)
    const error = ref<string | null>(null)

    function setError(message: string) {
        error.value = message
        isLoading.value = false
    }

    async function fetchItems(listId: number) {
        isLoading.value = true
        error.value = null
        try {
            const endpoint = API_ENDPOINTS.LISTS.ITEMS(String(listId))
            const { data } = await apiClient.get(endpoint)
            itemsByList.value[listId] = data as Item[]
        } catch (e: any) {
            setError(e?.response?.data?.detail || e.message || 'Failed to load items')
        } finally {
            isLoading.value = false
        }
    }

    async function addItem(payload: CreateItemPayload) {
        const { listId, data } = payload
        try {
            const endpoint = API_ENDPOINTS.LISTS.ITEMS(String(listId))
            const { data: created } = await apiClient.post(endpoint, data)
            if (!itemsByList.value[listId]) itemsByList.value[listId] = []
            itemsByList.value[listId].push(created as Item)
            return created as Item
        } catch (e: any) {
            setError(e?.response?.data?.detail || e.message || 'Failed to add item')
            throw e
        }
    }

    async function updateItem(payload: UpdateItemPayload) {
        const { listId, itemId, data } = payload
        try {
            const endpoint = API_ENDPOINTS.LISTS.ITEM(String(listId), String(itemId))
            const { data: updated } = await apiClient.put(endpoint, data)
            const arr = itemsByList.value[listId]
            if (arr) {
                const idx = arr.findIndex((it) => it.id === itemId)
                if (idx !== -1) arr[idx] = updated as Item
            }
            return updated as Item
        } catch (e: any) {
            setError(e?.response?.data?.detail || e.message || 'Failed to update item')
            throw e
        }
    }

    async function deleteItem(listId: number, itemId: number, expectedVersion?: number) {
        try {
            const endpoint = API_ENDPOINTS.LISTS.ITEM(String(listId), String(itemId))
            const config = expectedVersion ? { params: { expected_version: expectedVersion } } : {}
            await apiClient.delete(endpoint, config)
            const arr = itemsByList.value[listId]
            if (arr) itemsByList.value[listId] = arr.filter((it) => it.id !== itemId)
        } catch (e: any) {
            setError(e?.response?.data?.detail || e.message || 'Failed to delete item')
            throw e
        }
    }

    return {
        itemsByList,
        isLoading,
        error,
        fetchItems,
        addItem,
        updateItem,
        deleteItem,
        setError,
    }
}) 