import { defineStore } from 'pinia';
import { ref, computed } from 'vue';
import { apiClient, API_ENDPOINTS } from '@/services/api';
import type { List } from '@/types/list';
import type { Item } from '@/types/item';
import type { Expense, ExpenseSplit, SettlementActivityCreate } from '@/types/expense';
import { useAuthStore } from './auth';
import { API_BASE_URL } from '@/config/api-config';

export interface ListWithDetails extends List {
    items: Item[];
    expenses: Expense[];
}

export const useListsStore = defineStore('lists', () => {
    // --- STATE ---
    const currentList = ref<ListWithDetails | null>(null);
    const isLoading = ref(false);
    const error = ref<string | null>(null);
    const isSettlingSplit = ref(false);
    const socket = ref<WebSocket | null>(null);

    // Properties to support legacy polling logic in ListDetailPage, will be removed later.
    const lastListUpdate = ref<string | null>(null);
    const lastItemCount = ref<number | null>(null);

    // Getters
    const items = computed(() => currentList.value?.items || []);
    const expenses = computed(() => currentList.value?.expenses || []);

    function getPaidAmountForSplit(splitId: number): number {
        let totalPaid = 0;
        if (currentList.value && currentList.value.expenses) {
            for (const expense of currentList.value.expenses) {
                const split = expense.splits.find((s) => s.id === splitId);
                if (split && split.settlement_activities) {
                    totalPaid = split.settlement_activities.reduce((sum, activity) => {
                        return sum + parseFloat(activity.amount_paid);
                    }, 0);
                    break;
                }
            }
        }
        return totalPaid;
    }

    // --- ACTIONS ---
    async function fetchListDetails(listId: string) {
        isLoading.value = true;
        error.value = null;
        try {
            const listResponse = await apiClient.get(API_ENDPOINTS.LISTS.BY_ID(listId));
            const itemsResponse = await apiClient.get(API_ENDPOINTS.LISTS.ITEMS(listId));
            const expensesResponse = await apiClient.get(API_ENDPOINTS.LISTS.EXPENSES(listId));

            currentList.value = {
                ...listResponse.data,
                items: itemsResponse.data,
                expenses: expensesResponse.data,
            };
            // Update polling properties
            lastListUpdate.value = listResponse.data.updated_at;
            lastItemCount.value = itemsResponse.data.length;
        } catch (err: any) {
            error.value = err.response?.data?.detail || 'Failed to fetch list details.';
            console.error(err);
        } finally {
            isLoading.value = false;
        }
    }

    async function claimItem(itemId: number) {
        const item = currentList.value?.items.find((i: Item) => i.id === itemId);
        if (!item || !currentList.value) return;

        const originalClaimedById = item.claimed_by_user_id;
        const authStore = useAuthStore();

        const userId = Number(authStore.user!.id);
        item.claimed_by_user_id = userId;
        item.claimed_by_user = { id: userId, name: authStore.user!.name, email: authStore.user!.email };
        item.claimed_at = new Date().toISOString();

        try {
            const response = await apiClient.post(`/items/${itemId}/claim`);
            item.version = response.data.version;
        } catch (err: any) {
            item.claimed_by_user_id = originalClaimedById;
            item.claimed_by_user = null;
            item.claimed_at = null;
            error.value = err.response?.data?.detail || 'Failed to claim item.';
        }
    }

    async function unclaimItem(itemId: number) {
        const item = currentList.value?.items.find((i: Item) => i.id === itemId);
        if (!item) return;

        const originalClaimedById = item.claimed_by_user_id;
        const originalClaimedByUser = item.claimed_by_user;
        const originalClaimedAt = item.claimed_at;

        item.claimed_by_user_id = null;
        item.claimed_by_user = null;
        item.claimed_at = null;

        try {
            const response = await apiClient.delete(`/items/${itemId}/claim`);
            item.version = response.data.version;
        } catch (err: any) {
            item.claimed_by_user_id = originalClaimedById;
            item.claimed_by_user = originalClaimedByUser;
            item.claimed_at = originalClaimedAt;
            error.value = err.response?.data?.detail || 'Failed to unclaim item.';
        }
    }

    async function settleExpenseSplit(payload: { expense_split_id: number; activity_data: SettlementActivityCreate }) {
        isSettlingSplit.value = true;
        error.value = null;
        try {
            const endpoint = `/financials/expense_splits/${payload.expense_split_id}/settle`;
            await apiClient.post(endpoint, payload.activity_data);
            if (currentList.value?.id) {
                await fetchListDetails(String(currentList.value.id));
            }
            return true;
        } catch (err: any) {
            error.value = err.response?.data?.detail || 'Failed to settle expense split.';
            return false;
        } finally {
            isSettlingSplit.value = false;
        }
    }

    function buildWsUrl(listId: number, token: string | null): string {
        const base = import.meta.env.DEV
            ? `ws://${location.host}/ws/lists/${listId}`
            : `${API_BASE_URL.replace(/^http/, 'ws')}/ws/lists/${listId}`;
        return token ? `${base}?token=${token}` : base;
    }

    function connectWebSocket(listId: number, token: string | null) {
        if (socket.value) {
            socket.value.close();
        }
        const url = buildWsUrl(listId, token);
        socket.value = new WebSocket(url);
        socket.value.onopen = () => console.debug(`[WS] Connected to list channel ${listId}`);

        socket.value.onmessage = (event) => {
            try {
                const data = JSON.parse(event.data);
                if (data.type === 'item_claimed') {
                    handleItemClaimed(data.payload);
                } else if (data.type === 'item_unclaimed') {
                    handleItemUnclaimed(data.payload);
                }
                // Handle other item events here
            } catch (e) {
                console.error('[WS] Failed to parse message:', e);
            }
        };

        socket.value.onclose = () => {
            console.debug(`[WS] Disconnected from list channel ${listId}`);
            socket.value = null;
        };

        socket.value.onerror = (e) => console.error('[WS] Error:', e);
    }

    function disconnectWebSocket() {
        if (socket.value) {
            socket.value.close();
        }
    }

    // --- WEBSOCKET HANDLERS ---
    function handleItemClaimed(payload: any) {
        if (!currentList.value) return;
        const item = currentList.value.items.find((i: Item) => i.id === payload.item_id);
        if (item) {
            item.claimed_by_user_id = payload.claimed_by.id;
            item.claimed_by_user = payload.claimed_by;
            item.claimed_at = payload.claimed_at;
            item.version = payload.version;
        }
    }

    function handleItemUnclaimed(payload: any) {
        const item = items.value.find((i: Item) => i.id === payload.item_id);
        if (item) {
            item.claimed_by_user_id = null;
            item.claimed_by_user = null;
            item.claimed_at = null;
            item.version = payload.version;
        }
    }

    async function addItem(listId: string | number, payload: { name: string; quantity?: string | null; category_id?: number | null }) {
        if (!currentList.value) return;
        error.value = null;
        try {
            // Optimistic: push placeholder item with temporary id (-Date.now())
            const tempId = -Date.now();
            const tempItem: Item = {
                id: tempId,
                name: payload.name,
                quantity: payload.quantity ?? null,
                is_complete: false,
                price: null,
                list_id: Number(listId),
                category_id: payload.category_id ?? null,
                created_at: new Date().toISOString(),
                updated_at: new Date().toISOString(),
                completed_at: null,
                version: 0,
                added_by_id: useAuthStore().user?.id ?? 0,
                completed_by_id: null,
            } as unknown as Item;
            currentList.value.items.push(tempItem);

            const response = await apiClient.post(API_ENDPOINTS.LISTS.ITEMS(String(listId)), payload);
            // Replace temp item with actual item
            const index = currentList.value.items.findIndex(i => i.id === tempId);
            if (index !== -1) {
                currentList.value.items[index] = response.data;
            } else {
                currentList.value.items.push(response.data);
            }
            // Update lastListUpdate etc.
            lastItemCount.value = currentList.value.items.length;
            lastListUpdate.value = response.data.updated_at;
            return response.data;
        } catch (err: any) {
            // Rollback optimistic
            currentList.value.items = currentList.value.items.filter(i => i.id >= 0);
            error.value = err.response?.data?.detail || 'Failed to add item.';
            throw err;
        }
    }

    async function updateItem(listId: string | number, itemId: number, updates: Partial<Item> & { version: number }) {
        if (!currentList.value) return;
        error.value = null;
        const item = currentList.value.items.find(i => i.id === itemId);
        if (!item) return;
        const original = { ...item };
        // Optimistic merge
        Object.assign(item, updates);
        try {
            const response = await apiClient.put(API_ENDPOINTS.LISTS.ITEM(String(listId), String(itemId)), updates);
            // ensure store item updated with server version
            const index = currentList.value.items.findIndex(i => i.id === itemId);
            if (index !== -1) {
                currentList.value.items[index] = response.data;
            }
            return response.data;
        } catch (err: any) {
            // rollback
            const index = currentList.value.items.findIndex(i => i.id === itemId);
            if (index !== -1) {
                currentList.value.items[index] = original;
            }
            error.value = err.response?.data?.detail || 'Failed to update item.';
            throw err;
        }
    }

    async function deleteItem(listId: string | number, itemId: number, version: number) {
        if (!currentList.value) return;
        error.value = null;
        const index = currentList.value.items.findIndex(i => i.id === itemId);
        if (index === -1) return;
        const [removed] = currentList.value.items.splice(index, 1);
        try {
            await apiClient.delete(`${API_ENDPOINTS.LISTS.ITEM(String(listId), String(itemId))}?expected_version=${version}`);
            lastItemCount.value = currentList.value.items.length;
            return true;
        } catch (err: any) {
            // rollback
            currentList.value.items.splice(index, 0, removed);
            error.value = err.response?.data?.detail || 'Failed to delete item.';
            throw err;
        }
    }

    return {
        currentList,
        items,
        expenses,
        isLoading,
        error,
        isSettlingSplit,
        lastListUpdate,
        lastItemCount,
        fetchListDetails,
        claimItem,
        unclaimItem,
        settleExpenseSplit,
        getPaidAmountForSplit,
        handleItemClaimed,
        handleItemUnclaimed,
        connectWebSocket,
        disconnectWebSocket,
        addItem,
        updateItem,
        deleteItem,
    };
}); 